<?php

/**
 * @file
 * Contains CompressProviderZip.
 */

/**
 * Implements Zip compressor for File Compressor field.
 */
class CompressProviderZip implements CompressProviderInterface {

  /**
   * The extension for files using this plugin.
   *
   * @var string
   */
  protected $extension = 'zip';

  /**
   * {@inheritdoc}
   */
  public function getExtension() {
    return $this->extension;
  }

  /**
   * {@inheritdoc}
   */
  public function generateCompressedFileUri($base_uri) {
    return "$base_uri.$this->extension";
  }

  /**
   * {@inheritdoc}
   */
  public function generateCompressedFile($file_uri, $files) {
    $method = variable_get('file_compressor_field_zip_method', 'php');

    // Use PHP's ZipArchive class.
    if ($method === 'php') {
      $zip = new ZipArchive();
      if ($zip_open = $zip->open(drupal_realpath($file_uri), ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE)) {
        $zip->setArchiveComment('Generated by File Compressor field for Drupal.');
        // http://drupal.org/node/1234282 - duplicate filenames.
        foreach ($files as $file) {
          $path = drupal_realpath($file);
          if (file_exists($file)) {
            if (!$zip->addFile($path, basename($path))) {
              watchdog('file_compressor_field', 'Failed to add !file to Zip.', array('!file' => $file), WATCHDOG_ERROR);
            }
          }
          else {
            watchdog('file_compressor_field', 'Failed to locate !file.', array('!file' => $file), WATCHDOG_ERROR);
          }
        }
        if (!$zip->close()) {
          watchdog('file_compressor_field', 'Error saving file: !status_string', array('!status_string' => $zip->getStatusString()), WATCHDOG_ERROR);
          return FALSE;
        }
        return TRUE;
      }
    }

    // System / command line.
    elseif ($method === 'system') {
      // Build command.
      $command = 'zip -j ' . escapeshellarg(drupal_realpath($file_uri));
      foreach ($files as $file) {
        $command .= ' ' . escapeshellarg(drupal_realpath($file));
      }
      // Execute command.
      exec($command, $output, $return);
      if ($return !== 0) {
        watchdog('file_compressor_field', 'Error saving file %file. Command: %command ...failed with return status %return. Output: %output', array(
          '%file' => $file,
          '%command' => $command,
          '%return' => $return,
          '%output' => implode("\n", $output),
        ), WATCHDOG_ERROR);
        return FALSE;
      }
      return TRUE;
    }

    return FALSE;
  }

}
